<?xml version="1.0" encoding="UTF-8" ?>
<!-- Copyright (c) 2014 Veracode, Inc. All rights observed. Available for 
	use by Veracode customers as described in the accompanying license agreement. 
	Send bug reports or enhancement requests to support@veracode.com. See the 
	license agreement for conditions on submitted materials. -->
<xsl:stylesheet version="2.0"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:fn="http://www.w3.org/2005/xpath-functions">
	<!--Go to path /detailedreport/severity/category/cwe in DetailedReport.xml, 
		below that you will find either manualflaws or dynamicflaws or staticflaws 
		tag. 1. for manualflaws, refer"ManualFlaw" template mentioned below. 2. for 
		staticflaws, refer "StaticFlaw" template mentioned below. -->
	<xsl:template match="/">
		<xsl:element name="issues">
			<xsl:apply-templates
				select="/detailedreport/severity/category/cwe/manualflaws/flaw" />
			<xsl:apply-templates
				select="/detailedreport/severity/category/cwe/staticflaws/flaw" />
			<xsl:apply-templates
				select="/detailedreport/severity/category/cwe/dynamicflaws/flaw" />
		</xsl:element>
	</xsl:template>

	<!-- Manual Flaw Template: This will be used for every occurrence of /detailedreport/severity/category/cwe/manualflaws/flaw 
		in DetailedReport.xml And a JIRA issue will be created for every occurrence, 
		with following properties 1. project (mapped to detailedreport/app_name) 
		2. issueType ( set as Bug) 3. priority (mapped to flaw severity) 4. summary 
		(please refer ManualFlawSummary for mapping) 5. description (please refer 
		ManualFlawDescription for mapping)) 6. attachment (Applicable to manual flaws 
		and mapped to detailedreport/severity/category/cwe/manualflaws/flaw/appendix/screenshot/data) -->
	<xsl:template name="ManualFlaw"
		match="/detailedreport/severity/category/cwe/manualflaws/flaw">
		<xsl:for-each select=".">
			<!-- below mapping are inline with requirement document -->
			<xsl:element name="issue">
				<!-- Below are native properties of JIRA issue -->
				<xsl:element name="project">
					<xsl:call-template name="Project">
					</xsl:call-template>
				</xsl:element>
				<xsl:element name="issueType">
					<xsl:call-template name="IssueType">
					</xsl:call-template>
				</xsl:element>
				<xsl:element name="priority">
					<xsl:call-template name="Priority">
					</xsl:call-template>
				</xsl:element>
				<xsl:element name="summary">
					<xsl:call-template name="ManualFlawSummary">
					</xsl:call-template>
				</xsl:element>
				<xsl:element name="description">
					<xsl:call-template name="ManualFlawDescription">
					</xsl:call-template>
				</xsl:element>
				<xsl:element name="attachments">
					<xsl:call-template name="Attachments">
					</xsl:call-template>
				</xsl:element>
				<xsl:element name="affectedVersions">
					<xsl:value-of select="/detailedreport/@version" />
				</xsl:element>
				<xsl:element name="cweId">
					<xsl:value-of select="@cweid" />
				</xsl:element>
				<xsl:element name="mitigationStatus">
					<xsl:value-of select="@mitigation_status" />
				</xsl:element>
				<xsl:element name="mitigationStatusDescription">
					<xsl:value-of select="@mitigation_status_desc" />
				</xsl:element>
				<xsl:element name="affectsPolicyCompliance">
					<xsl:value-of select="@affects_policy_compliance" />
				</xsl:element>
				<xsl:element name="severity">
					<xsl:value-of select="@severity" />
				</xsl:element>
				<xsl:copy-of select="mitigations" />
				<xsl:copy-of select="annotations" />
				<!-- end of native properties -->
				<!-- Flaw/@issueid is mapped to custom property "issueId" in JIRA Issue -->
				<xsl:element name="issueId">
					<xsl:value-of select="@issueid" />
				</xsl:element>
				<xsl:element name="remediationStatus">
					<xsl:value-of select="@remediation_status" />
				</xsl:element>
				<xsl:call-template name="OtherJiraFields">
				</xsl:call-template>
				<!-- As many custom fields can be created for a JIRA Issue, refer below 
					"CustomFields" for details -->
				<xsl:element name="customFields">
					<xsl:call-template name="CustomFields">
					</xsl:call-template>
				</xsl:element>
			</xsl:element>
		</xsl:for-each>
	</xsl:template>

	<!-- Static Flaw Template: This will be used for every occurrence of /detailedreport/severity/category/cwe/staticflaws/flaw 
		in DetailedReport.xml And a JIRA issue will be created for every occurrence, 
		with following properties 1. project (mapped to detailedreport/@app_name) 
		2. issueType ( set as Bug) 3. priority (mapped to flaw severity, refer template 
		"Priority" for mapping) 4. summary (refer StaticFlawSummary for mapping) 
		5. description (refer StaticFlawDescription for mapping) -->
	<xsl:template name="StaticFlaw"
		match="/detailedreport/severity/category/cwe/staticflaws/flaw">
		<xsl:for-each select=".">
			<xsl:element name="issue">
				<xsl:element name="appName">
					<xsl:value-of select="/detailedreport/@app_name" />
				</xsl:element>
				<xsl:element name="publishedDate">
					<xsl:value-of select="/detailedreport/static-analysis/@published_date" />
				</xsl:element>
				<xsl:element name="submitter">
					<xsl:value-of select="/detailedreport/@submitter" />
				</xsl:element>
				<xsl:element name="lifeCycleStage">
					<xsl:value-of select="/detailedreport/@life_cycle_stage" />
				</xsl:element>
				<xsl:element name="policyName">
					<xsl:value-of select="/detailedreport/@policy_name" />
				</xsl:element>
				<xsl:element name="build_id">
					<xsl:value-of select="/detailedreport/@build_id" />
				</xsl:element>
				<xsl:element name="project">
					<xsl:call-template name="Project">
					</xsl:call-template>
				</xsl:element>
				<xsl:element name="issueType">
					<xsl:call-template name="IssueType">
					</xsl:call-template>
				</xsl:element>
				<xsl:element name="priority">
					<xsl:call-template name="Priority">
					</xsl:call-template>
				</xsl:element>
				<xsl:element name="summary">
					<xsl:call-template name="StaticFlawSummary">
					</xsl:call-template>
				</xsl:element>
				<xsl:element name="description">
					<xsl:call-template name="StaticFlawDescription">
					</xsl:call-template>
				</xsl:element>
				<xsl:element name="affectedVersions">
					<xsl:value-of select="/detailedreport/@version" />
				</xsl:element>
				<!-- Flaw/@issueid is mapped to custom property "issueId" in JIRA Issue -->
				<xsl:element name="issueId">
					<xsl:value-of select="@issueid" />
				</xsl:element>
				<xsl:element name="cweId">
					<xsl:value-of select="@cweid" />
				</xsl:element>
				<xsl:element name="mitigationStatus">
					<xsl:value-of select="@mitigation_status" />
				</xsl:element>
				<xsl:element name="mitigationStatusDescription">
					<xsl:value-of select="@mitigation_status_desc" />
				</xsl:element>
				<xsl:element name="affectsPolicyCompliance">
					<xsl:value-of select="@affects_policy_compliance" />
				</xsl:element>
				<xsl:element name="severity">
					<xsl:value-of select="@severity" />
				</xsl:element>
				<xsl:element name="remediationStatus">
					<xsl:value-of select="@remediation_status" />
				</xsl:element>
				<xsl:copy-of select="mitigations" />
				<xsl:copy-of select="annotations" />
				<xsl:call-template name="OtherJiraFields">
				</xsl:call-template>
				<!-- As many custom fields can be created for a JIRA Issue, refer below 
					"CustomFields" for details -->
				<xsl:element name="customFields">
					<xsl:call-template name="CustomFields">
					</xsl:call-template>
				</xsl:element>
			</xsl:element>

		</xsl:for-each>
	</xsl:template>
	<!-- ********************** Dynamic flaws ************************* -->
	<!-- Dynamic Flaw Template: This will be used for every occurrence of /detailedreport/severity/category/cwe/dynamicflaws/flaw 
		in DetailedReport.xml And a JIRA issue will be created for every occurrence, 
		with following properties 1. project (mapped to detailedreport/@app_name) 
		2. issueType ( set as Bug) 3. priority (mapped to flaw severity, refer template 
		"Priority" for mapping) 4. summary (refer DynamicFlawSummary for mapping) 
		5. description (refer DynamicFlawDescription for mapping) -->
	<xsl:template name="DynamicFlaw"
		match="/detailedreport/severity/category/cwe/dynamicflaws/flaw">
		<xsl:for-each select=".">
			<xsl:element name="issue">
				<xsl:element name="appName">
					<xsl:value-of select="/detailedreport/@app_name" />
				</xsl:element>
				<xsl:element name="publishedDate">
					<xsl:value-of select="/detailedreport/dynamic-analysis/@published_date" />
				</xsl:element>
				<xsl:element name="submitter">
					<xsl:value-of select="/detailedreport/@submitter" />
				</xsl:element>
				<xsl:element name="lifeCycleStage">
					<xsl:value-of select="/detailedreport/@life_cycle_stage" />
				</xsl:element>
				<xsl:element name="policyName">
					<xsl:value-of select="/detailedreport/@policy_name" />
				</xsl:element>
				<xsl:element name="build_id">
					<xsl:value-of select="/detailedreport/@build_id" />
				</xsl:element>
				<xsl:element name="project">
					<xsl:call-template name="Project">
					</xsl:call-template>
				</xsl:element>
				<xsl:element name="issueType">
					<xsl:call-template name="IssueType">
					</xsl:call-template>
				</xsl:element>
				<xsl:element name="priority">
					<xsl:call-template name="Priority">
					</xsl:call-template>
				</xsl:element>
				<xsl:element name="summary">
					<xsl:call-template name="DynamicFlawSummary">
					</xsl:call-template>
				</xsl:element>
				<xsl:element name="description">
					<xsl:call-template name="DynamicFlawDescription">
					</xsl:call-template>
				</xsl:element>
				<xsl:element name="affectedVersions">
					<xsl:value-of select="/detailedreport/@version" />
				</xsl:element>
				<!-- Flaw/@issueid is mapped to custom property "issueId" in JIRA Issue -->
				<xsl:element name="issueId">
					<xsl:value-of select="@issueid" />
				</xsl:element>
				<xsl:element name="cweId">
					<xsl:value-of select="@cweid" />
				</xsl:element>
				<xsl:element name="mitigationStatus">
					<xsl:value-of select="@mitigation_status" />
				</xsl:element>
				<xsl:element name="mitigationStatusDescription">
					<xsl:value-of select="@mitigation_status_desc" />
				</xsl:element>
				<xsl:element name="affectsPolicyCompliance">
					<xsl:value-of select="@affects_policy_compliance" />
				</xsl:element>
				<xsl:element name="severity">
					<xsl:value-of select="@severity" />
				</xsl:element>
				<xsl:element name="remediationStatus">
					<xsl:value-of select="@remediation_status" />
				</xsl:element>
				<xsl:copy-of select="mitigations" />
				<xsl:copy-of select="annotations" />
				<xsl:call-template name="OtherJiraFields">
				</xsl:call-template>
				<!-- As many custom fields can be created for a JIRA Issue, refer below 
					"CustomFields" for details -->
				<xsl:element name="customFields">
					<xsl:call-template name="CustomFields">
					</xsl:call-template>
				</xsl:element>
			</xsl:element>

		</xsl:for-each>
	</xsl:template>


	<!-- ************************** Dynamice flaws END *********************** -->


	<!-- The text fields below must match the JIRA priorities for your current 
		version. The default values below ("Highest", "High", ect) correspond to 
		the default values of JIRA 6. If your priorities differ for any reason, then 
		the below values must be changed. flaw/@severity is mapped to "Priority" 
		property of Jira Issue. Below rule is applicable for every occurrence of 
		/detailedreport/severity/category/cwe/staticflaws/flaw/@severity, or /detailedreport/severity/category/cwe/manualflaws/flaw/@severity 
		if Severity>4, Bug priority will be set "Highest" if Severity>3, Bug priority 
		will be set "High" if Severity>2, Bug priority will be set "Medium" if Severity<=2, 
		Bug priority will be set "Lowest" -->
	<xsl:template name="Priority">
		<xsl:choose>
			<xsl:when test="@severity &gt; 4">
				<xsl:text>Highest</xsl:text>
			</xsl:when>
			<xsl:when test="@severity &gt; 3">
				<xsl:text>High</xsl:text>
			</xsl:when>
			<xsl:when test="@severity &gt; 2">
				<xsl:text>Medium</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:text>Lowest</xsl:text>
			</xsl:otherwise>
		</xsl:choose>

	</xsl:template>

	<!-- ManualFlawSummary mapped to "Summary" property of JIRA bug for every 
		occurrence of /detailedreport/severity/category/cwe/manualflaws/flaw, Summary 
		= Veracode Flaw (manual): /detailedreport/severity/category/@categoryname, 
		/detailedreport/@app_name, /detailedreport/@version, /detailedreport/severity/category/cwe/manualflaws/flaw/@issueid -->
	<xsl:template name="ManualFlawSummary">
		<xsl:if test="/detailedreport/@sandbox_name">
			<xsl:text>Sandbox: </xsl:text>
			<xsl:value-of select="/detailedreport/@sandbox_name" />
			<xsl:text> / </xsl:text>
		</xsl:if>
		<xsl:text>Veracode Flaw (manual): </xsl:text>
		<xsl:value-of select="../../../@categoryname" /> <!-- /detailedreport/severity/category/@categoryname -->
		<xsl:text>, </xsl:text>
		<xsl:value-of select="/detailedreport/@app_name" />
		<xsl:text>, </xsl:text>
		<xsl:value-of select="/detailedreport/manual-analysis/@version" />
		<xsl:text>, Flaw </xsl:text>
		<xsl:value-of select="@issueid" />
	</xsl:template>

	<!-- StaticFlawSummary mapped to "Summary" property of JIRA bug for every 
		occurrence of /detailedreport/severity/category/cwe/staticflaws/flaw, Summary 
		= Veracode Flaw (static): /detailedreport/severity/category/@categoryname, 
		/detailedreport/@app_name, /detailedreport/@version, /detailedreport/severity/category/cwe/staticflaws/flaw/@issueid -->
	<xsl:template name="StaticFlawSummary">
		<xsl:if test="/detailedreport/@sandbox_name">
			<xsl:text>Sandbox: </xsl:text>
			<xsl:value-of select="/detailedreport/@sandbox_name" />
			<xsl:text> / </xsl:text>
		</xsl:if>
		<xsl:text>Veracode Flaw (static): </xsl:text>
		<xsl:value-of select="../../../@categoryname" />
		<xsl:text>, </xsl:text>
		<xsl:value-of select="/detailedreport/@app_name" />
		<xsl:text>, </xsl:text>
		<xsl:value-of select="/detailedreport/static-analysis/@version" />
		<xsl:text>, Flaw </xsl:text>
		<xsl:value-of select="@issueid" />
	</xsl:template>


	<!-- ************************DynamicFlawSummary ********************** -->
	<!-- DynamicFlawSummary mapped to "Summary" property of JIRA bug for every 
		occurrence of /detailedreport/severity/category/cwe/dynamicflaws/flaw, Summary 
		= Veracode Flaw (dynamic): /detailedreport/severity/category/@categoryname, 
		/detailedreport/@app_name, /detailedreport/@version, /detailedreport/severity/category/cwe/dynamicflaws/flaw/@issueid -->
	<xsl:template name="DynamicFlawSummary">
		<xsl:if test="/detailedreport/@sandbox_name">
			<xsl:text>Sandbox: </xsl:text>
			<xsl:value-of select="/detailedreport/@sandbox_name" />
			<xsl:text> / </xsl:text>
		</xsl:if>
		<xsl:text>Veracode Flaw (dynamic): </xsl:text>
		<xsl:value-of select="../../../@categoryname" />
		<xsl:text>, </xsl:text>
		<xsl:value-of select="/detailedreport/@app_name" />
		<xsl:text>, </xsl:text>
		<xsl:value-of select="/detailedreport/dynamic-analysis/@version" />
		<xsl:text>, Flaw </xsl:text>
		<xsl:value-of select="@issueid" />
	</xsl:template>
	<!-- *********************DynamicFlawSummary End************************** -->

	<!-- IssueType mapped to "issueType" property of JIRA bug. -->
	<xsl:template name="IssueType">
		<xsl:text>Bug</xsl:text>
	</xsl:template>

	<!-- /detailedreport/@app_name is mapped to "project" property of JIRA bug. -->
	<xsl:template name="Project">
		<xsl:text>VER</xsl:text>
		<!-- <xsl:value-of select="/detailedreport/@app_name"/> -->
	</xsl:template>

	<!-- ManualFlawDescription mapped to "Description" property of JIRA bug 
		for every occurrence of /detailedreport/severity/category/cwe/manualflaws/flaw, 
		Description = CWE:/detailedreport/severity/category/cwe/manualflaws/flaw/@cweid 
		(it will be hyper link, linking to http://cwe.mitre.org/data/definitions/<cweid>.html) 
		Location: /detailedreport/severity/category/cwe/manualflaws/flaw/@location, 
		Description: /detailedreport/severity/category/cwe/manualflaws/flaw/@description, 
		Exploitability: /detailedreport/severity/category/cwe/manualflaws/flaw/exploit_desc 
		Severity: /detailedreport/severity/category/cwe/manualflaws/flaw/severity_desc 
		Remediation: /detailedreport/severity/category/cwe/manualflaws/flaw/remediation_desc 
		Code Sample: /detailedreport/severity/category/cwe/manualflaws/flaw/appendix/code -->

	<xsl:template name="ManualFlawDescription">
		<xsl:variable name="cwelink">
			<xsl:text>http://cwe.mitre.org/data/definitions/</xsl:text>
			<xsl:value-of select="@cweid" />
			<xsl:text>.html</xsl:text>
		</xsl:variable>
		<xsl:if test="/detailedreport/@sandbox_name">
			<xsl:text> *Sandbox:* </xsl:text>
			<xsl:value-of select="/detailedreport/@sandbox_name" />
			<xsl:text>\\ </xsl:text>
			<xsl:text>\\ </xsl:text>
		</xsl:if>
		<xsl:text> *CWE:* </xsl:text>
		[
		<xsl:value-of select="@cweid" />
		|
		<xsl:value-of select="$cwelink" />
		]
		<xsl:value-of select="@categoryname" />
		<xsl:text>\\ </xsl:text>
		<xsl:text>\\ </xsl:text>
		<xsl:text> *Location:* </xsl:text>
		<xsl:value-of select="@location" />
		<xsl:text>\\ </xsl:text>
		<xsl:text>\\ </xsl:text>
		<xsl:text> *Description:* </xsl:text>
		<xsl:call-template name="DescT"></xsl:call-template>
		<xsl:text>\\ </xsl:text>
		<xsl:text>\\ </xsl:text>
		<xsl:text> *Exploitability:* </xsl:text>
		<xsl:value-of select="exploit_desc" />
		<xsl:text>\\ </xsl:text>
		<xsl:text>\\ </xsl:text>
		<xsl:text> *Severity:* </xsl:text>
		<xsl:value-of select="severity_desc" />
		<xsl:text>\\ </xsl:text>
		<xsl:text>\\ </xsl:text>
		<xsl:text> *Remediation:* </xsl:text>
		<xsl:value-of select="remediation_desc" />
		<xsl:text>\\ </xsl:text>
		<xsl:text>\\ </xsl:text>
		<xsl:text> *Code Sample:* </xsl:text>
		<xsl:value-of select="appendix/code" />
	</xsl:template>


	<!-- StaticFlawDescription mapped to "Description" property of JIRA bug 
		for every occurrence of /detailedreport/severity/category/cwe/staticflaws/flaw, 
		Description = CWE:/detailedreport/severity/category/cwe/staticflaws/flaw/@cweid 
		(it will be hyper link, linking to http://cwe.mitre.org/data/definitions/<cweid>.html) 
		Module: /detailedreport/severity/category/cwe/staticflaws/flaw/@module, Source: 
		/detailedreport/severity/category/cwe/staticflaws/flaw/@sourcefile /detailedreport/severity/category/cwe/staticflaws/flaw/@line, 
		Attack Vector: /detailedreport/severity/category/cwe/staticflaws/flaw/@type 
		Description: /detailedreport/severity/category/cwe/staticflaws/flaw/@description -->

	<xsl:template name="StaticFlawDescription">
		<xsl:variable name="cwelink">
			<xsl:text>http://cwe.mitre.org/data/definitions/</xsl:text>
			<xsl:value-of select="@cweid" />
			<xsl:text>.html</xsl:text>
		</xsl:variable>
		<xsl:if test="/detailedreport/@sandbox_name">
			<!--xsl:text> *Sandbox:* </xsl:text><xsl:value-of select="/detailedreport/@sandbox_name"/> 
				<xsl:text>\\ </xsl:text> <xsl:text>\\ </xsl:text -->
		</xsl:if>
		<xsl:text>  CWE: </xsl:text><xsl:value-of select="@cweid" /> [<xsl:value-of select="$cwelink" /> ] <xsl:value-of select="@categoryname" />
		<xsl:text>\\ </xsl:text>
		<!--xsl:text>\\ </xsl:text -->
		<xsl:text> Module: </xsl:text>
		<xsl:value-of select="@module" />
		<xsl:text>\\ </xsl:text>
		<!--xsl:text>\\ </xsl:text -->
		<xsl:text> Source: </xsl:text>
		<xsl:value-of select="@sourcefile" />
		<xsl:text>:</xsl:text>
		<xsl:value-of select="@line" />
		<xsl:text>\\ </xsl:text>
		<!--xsl:text>\\ </xsl:text -->
		<xsl:text> Attack Vector: </xsl:text>
		<xsl:value-of select="@type" />
		<xsl:text>\\ </xsl:text>
		<!--xsl:text>\\ </xsl:text -->
		<xsl:text> Description: </xsl:text>
		<xsl:call-template name="DescT"></xsl:call-template>

	</xsl:template>

	<!-- ******************** DynamicFlawDescription start****************************** -->
	<!-- DynamicFlawDescription mapped to "Description" property of JIRA bug 
		for every occurrence of /detailedreport/severity/category/cwe/dynamicflaws/flaw, 
		Description = CWE:/detailedreport/severity/category/cwe/dynamicflaws/flaw/@cweid 
		(it will be hyper link, linking to http://cwe.mitre.org/data/definitions/<cweid>.html) 
		Module: /detailedreport/severity/category/cwe/dynamicflaws/flaw/@module, 
		Source: /detailedreport/severity/category/cwe/dynamicflaws/flaw/@sourcefile 
		/detailedreport/severity/category/cwe/dynamicflaws/flaw/@line, Attack Vector: 
		/detailedreport/severity/category/cwe/dynamicflaws/flaw/@type Description: 
		/detailedreport/severity/category/cwe/dynamicflaws/flaw/@description -->

	<xsl:template name="DynamicFlawDescription">
		<xsl:variable name="cwelink">
			<xsl:text>http://cwe.mitre.org/data/definitions/</xsl:text>
			<xsl:value-of select="@cweid" />
			<xsl:text>.html</xsl:text>
		</xsl:variable>
		<xsl:if test="/detailedreport/@sandbox_name">
			<!--xsl:text> *Sandbox:* </xsl:text><xsl:value-of select="/detailedreport/@sandbox_name"/> 
				<xsl:text>\\ </xsl:text> <xsl:text>\\ </xsl:text -->
		</xsl:if>
		<xsl:text>  CWE:  </xsl:text>
		<xsl:value-of select="@cweid" /> [ <xsl:value-of select="$cwelink" /> <xsl:value-of select="@categoryname" />
		<xsl:text>\\ </xsl:text>
		<!--xsl:text>\\ </xsl:text -->
		<xsl:text> URL: </xsl:text>
		<xsl:value-of select="@url" />
		<xsl:text> \\ </xsl:text>
		<!--xsl:text>\\ </xsl:text -->
		<xsl:text> Parameter: </xsl:text>
		<xsl:value-of select="@vuln_parameter" /><!--<xsl:text>:</xsl:text><xsl:value-of 
			select="@line"/> -->
		<xsl:text>\\ </xsl:text>
		<!--xsl:text>\\ </xsl:text -->

		<xsl:text> Description: </xsl:text>
		<xsl:call-template name="DescT"></xsl:call-template>
	</xsl:template>
	<!-- **********************DynamicFlawDescription End************************* -->

	<!-- This template checks flaw/@description, if it contains"References:", 
		then divides it 2 parts with 1 part before "References:", and 2nd part after 
		"References:". And formats the 2nd part (i.e. after "References:") to enable 
		Hyperlinks as per Jira format/style. -->
	<xsl:template name="DescT">
		<xsl:variable name="desc">
			<xsl:value-of select="@description" />
		</xsl:variable>
		<xsl:variable name="descFormatted">
			<xsl:choose>
				<xsl:when test="contains($desc, 'References:')">
					<xsl:value-of select="substring-before($desc, 'References:')" />
					<xsl:call-template name="RefT">
						<xsl:with-param name="var"
							select="substring-after($desc, 'References:')" />
					</xsl:call-template>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="$desc" />
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:value-of select="$descFormatted" />
	</xsl:template>

	<!-- This template takes the "References:", part and checks if it contains 
		"links" with format "(link)" eg: CWE (http://cwe.mitre.org/data/definitions/89.html), 
		then it will be displayed as CWE hyperlinked to appropriate page -->

	<xsl:template name="RefT">
		<xsl:param name="var" />
		<xsl:variable name="var1">
			<xsl:value-of select="normalize-space($var)" />
		</xsl:variable>
		<xsl:text> </xsl:text><!-- Changed for HPALM -->
		<xsl:text> References: </xsl:text>
		<xsl:choose>
			<xsl:when
				test="(contains($var1, 'http')) and (contains($var1, ')')) and contains($var1, '(')">
				<xsl:variable name="refToken">
					<xsl:value-of select="translate($var1, '()', '|]')" />
				</xsl:variable>
				<xsl:variable name="refFinal">
				</xsl:variable>
				<xsl:call-template name="refFormat">
					<xsl:with-param name="refTokenLength" select="string-length($refToken)" />
					<xsl:with-param name="refStr" select="$refToken" />
					<xsl:with-param name="refFinal" select="$refFinal" />
				</xsl:call-template>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$var1" />
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="refFormat">
		<xsl:param name="refTokenLength" />
		<xsl:param name="refStr" />
		<xsl:param name="refFinal" />

		<xsl:if test="($refTokenLength > 0)">
			<xsl:variable name="refTokenBef">
				<xsl:value-of select="substring-before($refStr, ']')" />
			</xsl:variable>
			<xsl:variable name="refTokenAft">
				<xsl:value-of select="substring-after($refStr, ']')" />
			</xsl:variable>
			<xsl:variable name="refToken3">
				<xsl:text> [</xsl:text>
				<xsl:value-of select="$refTokenBef" />
				<xsl:text>] </xsl:text>
			</xsl:variable>

			<xsl:variable name="refToken4">
				<xsl:value-of select="concat($refFinal, $refToken3 )" />
			</xsl:variable>
			<xsl:choose>
				<xsl:when test="(string-length($refTokenAft) > 0)">
					<xsl:call-template name="refFormat">
						<xsl:with-param name="refTokenLength" select="string-length($refTokenAft)" />
						<xsl:with-param name="refStr"
							select="normalize-space($refTokenAft)" />
						<xsl:with-param name="refFinal" select="$refToken4" />
					</xsl:call-template>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="$refToken4" />
				</xsl:otherwise>
			</xsl:choose>
		</xsl:if>
	</xsl:template>

	<!--Attachment mapped to "Attachment" property of JIRA Bug. Attachment can 
		be traced at every occurrence of detailedreport/severity/category/cwe/manualflaws/flaw/appendix/screenshot/data 
		2 properties of screenshot i.e. data and format attribute are in use. It 
		is applicable to manual flaws. ) -->
	<xsl:template name="Attachments">
		<xsl:for-each select="appendix/screenshot">
			<xsl:element name="attachment">
				<xsl:element name="base64EncodedData">
					<xsl:value-of select="data" />
				</xsl:element>
				<xsl:element name="extension">
					<xsl:value-of select="@format" />
				</xsl:element>
			</xsl:element>
		</xsl:for-each>
	</xsl:template>

	<!--Other Jira native fields can be mapped here -->
	<xsl:template name="OtherJiraFields">
		<xsl:element name="assignee"></xsl:element> <!-- valid Jira user name, and should have access to choosen project, else 
			default project assignee will be considered -->
		<xsl:element name="reporter"></xsl:element> <!-- valid Jira user name, and should have access to choosen project -->
		<xsl:element name="environment">
			environment
		</xsl:element>
		<xsl:element name="parentId"></xsl:element> <!-- Should be of datatype:: long -->
		<xsl:element name="resolution"></xsl:element><!-- Native Jira "resolution" 
			values: Fixed or Won't Fix or Duplicate or Incomplete or Cannot Reproduce -->
		<xsl:element name="timeSpent"></xsl:element><!-- Should be of datatype: 
			long -->
		<xsl:element name="status">
			open
		</xsl:element> <!-- Native Jira "status" values:: Open or In Progress or Reopened or Resolved 
			or Closed -->
		<xsl:element name="fixVersions"></xsl:element><!-- "," separated versions. 
			If given versions doesn't exist in given project, then versions will be added 
			to given project. -->
		<xsl:element name="components">
		</xsl:element> <!--"," separated component lists. If given components doesn't exist in given 
			project, then it will be added to given project. <compName1>:<compDesc1>:<compLeadName1>:<assigneeType2>,<compName2>:<compDesc2>:<compLeadName2>:<assigneeType2> 
			eg: c1:c1 desc:sunita:0, c2: c2 desc: sunita : 0 If component lead not given, 
			default project assignee will be taken If AssigneeType not given, 0(PROJECT_DEFAULT) 
			will be taken if no value found for a component part, atlease a space should 
			be given. eg: c3: : : -->
		<xsl:element name="dateCreated"></xsl:element> <!-- should be of format "yyyy-MM-dd HH:mm:ss:SSS" eg: "2012-11-13 14:04:59:612" -->
		<xsl:element name="dueDate"></xsl:element><!-- should be of format "yyyy-MM-dd 
			HH:mm:ss:SSS" eg: "2012-11-13 14:04:59:612" -->
		<xsl:element name="resolutionDate"></xsl:element><!-- should be of format "yyyy-MM-dd 
			HH:mm:ss:SSS" eg: "2012-11-13 14:04:59:612" -->
		<xsl:element name="updatedDate"></xsl:element><!-- should be of format "yyyy-MM-dd 
			HH:mm:ss:SSS" eg: "2012-11-13 14:04:59:612" -->
		<xsl:element name="estimate"></xsl:element> <!-- Should be of datatype: long -->
		<xsl:element name="originalEstimate"></xsl:element><!-- Should be of datatype:: 
			long -->
		<xsl:element name="votes"></xsl:element><!-- Should be of datatype:: 
			long -->
	</xsl:template>

	<!--To map any flaw property to a custom field, make sure the custom field 
		is created in Jira. To create any flaw property as custom field, below mapping 
		should be set. A CustomField can have below properties, 1. cfName : name 
		of custom field 2. cfType : datatype of custom field 3. cfValues (list of 
		values) : possible values of custom field So far below are few examples given 
		as custom fields. 1. cwe id : mapped with /detailedreport/severity/category/cwe/@cweid 
		2. cwe name: mapped with /detailedreport/severity/category/cwe/@cwename 3. 
		module: mapped with /detailedreport/severity/category/cwe/staticflaws/flaw/@module 
		4. source file: mapped with /detailedreport/severity/category/cwe/staticflaws/flaw/@sourcefile 
		5. line: mapped with /detailedreport/severity/category/cwe/staticflaws/flaw/@line 
		6. sans: mapped with /detailedreport/severity/category/cwe/@sans 7. owasp: 
		mapped with /detailedreport/severity/category/cwe/@owasp -->
	<xsl:template name="CustomFields">
		<!-- <xsl:element name="customField"> <xsl:element name="cfName">cweid</xsl:element> 
			<xsl:element name="cfType">com.atlassian.jira.plugin.system.customfieldtypes:textfield</xsl:element> 
			<xsl:element name="cfValues"><xsl:value-of select="../../@cweid"/></xsl:element> 
			</xsl:element> -->
	</xsl:template>

</xsl:stylesheet>
